@@ -1,7 +1,7 @@
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2002 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -30,10 +30,11 @@
 #include "mdadm.h"
 #include	"md_u.h"
 #include	"md_p.h"
+#include	<ctype.h>
 
 int Create(struct supertype *st, char *mddev, int mdfd,
 	   int chunk, int level, int layout, unsigned long long size, int raiddisks, int sparedisks,
-	   char *name,
+	   char *name, char *homehost,
 	   int subdevs, mddev_dev_t devlist,
 	   int runstop, int verbose, int force, int assume_clean,
 	   char *bitmap_file, int bitmap_chunk, int write_behind, int delay)
@@ -214,6 +215,11 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 			ldsize = dsize;
 			ldsize <<= 9;
 		}
+		if (st == NULL) {
+			struct createinfo *ci = conf_get_create_info();
+			if (ci)
+				st = ci->supertype;
+		}
 		if (st == NULL) {
 			/* Need to choose a default metadata, which is different
 			 * depending on the sizes of devices
@@ -389,7 +395,31 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 	array.chunk_size = chunk*1024;
 	array.major_version = st->ss->major;
 
-	if (!st->ss->init_super(st, &super, &array, size, name))
+	if (name == NULL || *name == 0) {
+		/* base name on mddev */
+		/*  /dev/md0 -> 0
+		 *  /dev/md_d0 -> d0
+		 *  /dev/md/1 -> 1
+		 *  /dev/md/d1 -> d1
+		 *  /dev/md/home -> home
+		 *  /dev/mdhome -> home
+		 */
+		name = strrchr(mddev, '/');
+		if (name) {
+			name++;
+			if (strncmp(name, "md_d", 4)==0 &&
+			    strlen(name) > 4 &&
+			    isdigit(name[4]) &&
+			    (name-mddev) == 5 /* /dev/ */)
+				name += 3;
+			else if (strncmp(name, "md", 2)==0 &&
+				 strlen(name) > 2 &&
+				 isdigit(name[2]) &&
+				 (name-mddev) == 5 /* /dev/ */)
+				name += 2;
+		}
+	}
+	if (!st->ss->init_super(st, &super, &array, size, name, homehost))
 		return 1;
 
 	if (bitmap_file && vers < 9003) {
@@ -432,9 +462,6 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 	if (bitmap_file) {
 		int uuid[4];
 
-		if (bitmap_chunk == UnSet)
-			bitmap_chunk = DEFAULT_BITMAP_CHUNK;
-
 		st->ss->uuid_from_super(uuid, super);
 		if (CreateBitmap(bitmap_file, force, (char*)uuid, bitmap_chunk,
 				 delay, write_behind,
@@ -494,6 +521,7 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 				fstat(fd, &stb);
 				disk.major = major(stb.st_rdev);
 				disk.minor = minor(stb.st_rdev);
+				remove_partitions(fd);
 				close(fd);
 			}
 			switch(pass){
@@ -502,6 +530,8 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 				break;
 			case 2:
 				if (disk.state == 1) break;
+				Kill(dv->devname, 0, 1); /* Just be sure it is clean */
+				Kill(dv->devname, 0, 1); /* and again, there could be two superblocks */
 				st->ss->write_init_super(st, super, &disk, dv->devname);
 
 				if (ioctl(mdfd, ADD_NEW_DISK, &disk)) {
