@@ -75,6 +75,7 @@ int main(int argc, char *argv[])
 	int force = 0;
 	int test = 0;
 	int assume_clean = 0;
+	char *symlinks = NULL;
 	/* autof indicates whether and how to create device node.
 	 * bottom 3 bits are style.  Rest (when shifted) are number of parts
 	 * 0  - unset
@@ -499,6 +500,12 @@ int main(int argc, char *argv[])
 			autof = parse_auto(optarg, "--auto flag", 0);
 			continue;
 
+		case O(CREATE,Symlinks):
+		case O(BUILD,Symlinks):
+		case O(ASSEMBLE,Symlinks): /* auto creation of symlinks in /dev to /dev/md */
+			symlinks = optarg;
+			continue;
+
 		case O(BUILD,'f'): /* force honouring '-n 1' */
 		case O(GROW,'f'): /* ditto */
 		case O(CREATE,'f'): /* force honouring of device list */
@@ -858,6 +865,19 @@ int main(int argc, char *argv[])
 		fputs(Usage, stderr);
 		exit(2);
 	}
+
+	if (symlinks) {
+		struct createinfo *ci = conf_get_create_info();
+
+		if (strcasecmp(symlinks, "yes") == 0)
+			ci->symlinks = 1;
+		else if (strcasecmp(symlinks, "no") == 0)
+			ci->symlinks = 0;
+		else {
+			fprintf(stderr, Name ": option --symlinks must be 'no' or 'yes'\n");
+			exit(2);
+		}
+	}
 	/* Ok, got the option parsing out of the way
 	 * hopefully it's mostly right but there might be some stuff
 	 * missing
@@ -1017,7 +1037,7 @@ int main(int argc, char *argv[])
 				else {
 					rv |= Assemble(ss, array_list->devname, mdfd,
 						       array_list,
-						       devlist, NULL,
+						       NULL, NULL,
 						       readonly, runstop, NULL, homehost, verbose-quiet, force);
 					if (rv == 0) cnt++;
 				}
@@ -1036,7 +1056,7 @@ int main(int argc, char *argv[])
 					do {
 						rv2 = Assemble(ss, NULL, -1,
 							       &ident,
-							       devlist, NULL,
+							       NULL, NULL,
 							       readonly, runstop, NULL, homehost, verbose-quiet, force);
 						if (rv2==0) {
 							cnt++;
@@ -1049,7 +1069,6 @@ int main(int argc, char *argv[])
 							auto_update_home = 0;
 					} while (rv2!=2);
 					/* Incase there are stacked devices, we need to go around again */
-					devlist = conf_get_devs();
 				} while (acnt);
 				if (cnt == 0 && auto_update_home && homehost) {
 					/* Nothing found, maybe we need to bootstrap homehost info */
@@ -1058,7 +1077,7 @@ int main(int argc, char *argv[])
 						do {
 							rv2 = Assemble(ss, NULL, -1,
 								       &ident,
-								       devlist, NULL,
+								       NULL, NULL,
 								       readonly, runstop, "homehost", homehost, verbose-quiet, force);
 							if (rv2==0) {
 								cnt++;
@@ -1066,7 +1085,6 @@ int main(int argc, char *argv[])
 							}
 						} while (rv2!=2);
 						/* Incase there are stacked devices, we need to go around again */
-						devlist = conf_get_devs();
 					} while (acnt);
 				}
 				if (cnt == 0 && rv == 0) {
