@@ -1,7 +1,7 @@
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2004 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -104,7 +104,7 @@ struct mdp_superblock_1 {
 #define	MD_FEATURE_ALL			(1|2|4)
 
 #ifndef offsetof
-#define offsetof(t,f) ((int)&(((t*)0)->f))
+#define offsetof(t,f) ((size_t)&(((t*)0)->f))
 #endif
 static unsigned int calc_sb_1_csum(struct mdp_superblock_1 * sb)
 {
@@ -124,19 +124,21 @@ static unsigned int calc_sb_1_csum(struct mdp_superblock_1 * sb)
 	disk_csum = sb->sb_csum;
 	sb->sb_csum = 0;
 	newcsum = 0;
-	for (i=0; size>=4; size -= 4 )
-		newcsum += __le32_to_cpu(*isuper++);
+	for (i=0; size>=4; size -= 4 ) {
+		newcsum += __le32_to_cpu(*isuper);
+		isuper++;
+	}
 
 	if (size == 2)
 		newcsum += __le16_to_cpu(*(unsigned short*) isuper);
 
 	csum = (newcsum & 0xffffffff) + (newcsum >> 32);
 	sb->sb_csum = disk_csum;
-	return csum;
+	return __cpu_to_le32(csum);
 }
 
 #ifndef MDASSEMBLE
-static void examine_super1(void *sbv)
+static void examine_super1(void *sbv, char *homehost)
 {
 	struct mdp_superblock_1 *sb = sbv;
 	time_t atime;
@@ -144,6 +146,8 @@ static void examine_super1(void *sbv)
 	int faulty;
 	int i;
 	char *c;
+	int l = homehost ? strlen(homehost) : 0;
+	int layout;
 
 	printf("          Magic : %08x\n", __le32_to_cpu(sb->magic));
 	printf("        Version : %02d\n", 1);
@@ -154,8 +158,12 @@ static void examine_super1(void *sbv)
 		printf("%02x", sb->set_uuid[i]);
 	}
 	printf("\n");
-	printf("           Name : %.32s\n", sb->set_name);
-
+	printf("           Name : %.32s", sb->set_name);
+	if (l > 0 && l < 32 &&
+	    sb->set_name[l] == ':' &&
+	    strncmp(sb->set_name, homehost, l) == 0)
+		printf("  (local to host %s)", homehost);
+	printf("\n");
 	atime = __le64_to_cpu(sb->ctime) & 0xFFFFFFFFFFULL;
 	printf("  Creation Time : %.24s\n", ctime(&atime));
 	c=map_num(pers, __le32_to_cpu(sb->level));
@@ -168,14 +176,21 @@ static void examine_super1(void *sbv)
 		switch(__le32_to_cpu(sb->level)) {
 		case 1: ddsks=1;break;
 		case 4:
-		case 5: ddsks = sb->raid_disks-1; break;
-		case 6: ddsks = sb->raid_disks-2; break;
-		case 10: ddsks = sb->raid_disks / (sb->layout&255) / ((sb->layout>>8)&255);
+		case 5: ddsks = __le32_to_cpu(sb->raid_disks)-1; break;
+		case 6: ddsks = __le32_to_cpu(sb->raid_disks)-2; break;
+		case 10:
+			layout = __le32_to_cpu(sb->layout);
+			ddsks = __le32_to_cpu(sb->raid_disks)
+				 / (layout&255) / ((layout>>8)&255);
 		}
 		if (ddsks)
-			printf("     Array Size : %llu%s\n", ddsks*(unsigned long long)sb->size, human_size(ddsks*sb->size<<9));
+			printf("     Array Size : %llu%s\n",
+			       ddsks*(unsigned long long)__le64_to_cpu(sb->size),
+			       human_size(ddsks*__le64_to_cpu(sb->size)<<9));
 		if (sb->size != sb->data_size)
-			printf("      Used Size : %llu%s\n", (unsigned long long)sb->size, human_size(sb->size<<9));
+			printf("      Used Size : %llu%s\n",
+			       (unsigned long long)__le64_to_cpu(sb->size),
+			       human_size(__le64_to_cpu(sb->size)<<9));
 	}
 	if (sb->data_offset)
 		printf("    Data Offset : %llu sectors\n", (unsigned long long)__le64_to_cpu(sb->data_offset));
@@ -218,8 +233,9 @@ static void examine_super1(void *sbv)
 				printf("     New Layout : %s\n", c?c:"-unknown-");
 			}
 			if (__le32_to_cpu(sb->level) == 10) {
-				printf("     New Layout : near=%d, far=%d\n",
+				printf("     New Layout : near=%d, %s=%d\n",
 				       __le32_to_cpu(sb->new_layout)&255,
+				       (__le32_to_cpu(sb->new_layout)&0x10000)?"offset":"far",
 				       (__le32_to_cpu(sb->new_layout)>>8)&255);
 			}
 		}
@@ -250,8 +266,10 @@ static void examine_super1(void *sbv)
 	}
 	if (__le32_to_cpu(sb->level) == 10) {
 		int lo = __le32_to_cpu(sb->layout);
-		printf("         Layout : near=%d, far=%d\n",
-		       lo&255, (lo>>8)&255);
+		printf("         Layout : near=%d, %s=%d\n",
+		       lo&255,
+		       (lo&0x10000)?"offset":"far",
+		       (lo>>8)&255);
 	}
 	switch(__le32_to_cpu(sb->level)) {
 	case 0:
@@ -267,6 +285,18 @@ static void examine_super1(void *sbv)
 	default: break;
 	}
 	printf("\n");
+	printf("    Array Slot : %d (", __le32_to_cpu(sb->dev_number));
+	for (i= __le32_to_cpu(sb->max_dev); i> 0 ; i--)
+		if (__le16_to_cpu(sb->dev_roles[i-1]) != 0xffff)
+			break;
+	for (d=0; d < i; d++) {
+		int role = __le16_to_cpu(sb->dev_roles[d]);
+		if (d) printf(", ");
+		if (role == 0xffff) printf("empty");
+		else if(role == 0xfffe) printf("failed");
+		else printf("%d", role);
+	}
+	printf(")\n");
 	printf("   Array State : ");
 	for (d=0; d<__le32_to_cpu(sb->raid_disks); d++) {
 		int cnt = 0;
@@ -300,11 +330,20 @@ static void brief_examine_super1(void *sbv)
 {
 	struct mdp_superblock_1 *sb = sbv;
 	int i;
-
+	char *nm;
 	char *c=map_num(pers, __le32_to_cpu(sb->level));
 
-	printf("ARRAY /dev/?? level=%s metadata=1 num-devices=%d UUID=",
-	       c?c:"-unknown-", sb->raid_disks);
+	nm = strchr(sb->set_name, ':');
+	if (nm)
+		nm++;
+	else if (sb->set_name[0])
+		nm = sb->set_name;
+	else
+		nm = "??";
+
+	printf("ARRAY /dev/md/%s level=%s metadata=1 num-devices=%d UUID=",
+	       nm,
+	       c?c:"-unknown-", __le32_to_cpu(sb->raid_disks));
 	for (i=0; i<16; i++) {
 		printf("%02x", sb->set_uuid[i]);
 		if ((i&3)==0 && i != 0) printf(":");
@@ -314,13 +353,18 @@ static void brief_examine_super1(void *sbv)
 	printf("\n");
 }
 
-static void detail_super1(void *sbv)
+static void detail_super1(void *sbv, char *homehost)
 {
 	struct mdp_superblock_1 *sb = sbv;
 	int i;
+	int l = homehost ? strlen(homehost) : 0;
 
-	printf("           Name : %.32s\n", sb->set_name);
-	printf("           UUID : ");
+	printf("           Name : %.32s", sb->set_name);
+	if (l > 0 && l < 32 &&
+	    sb->set_name[l] == ':' &&
+	    strncmp(sb->set_name, homehost, l) == 0)
+		printf("  (local to host %s)", homehost);
+	printf("\n           UUID : ");
 	for (i=0; i<16; i++) {
 		if ((i&3)==0 && i != 0) printf(":");
 		printf("%02x", sb->set_uuid[i]);
@@ -344,6 +388,16 @@ static void brief_detail_super1(void *sbv)
 
 #endif
 
+static int match_home1(void *sbv, char *homehost)
+{
+	struct mdp_superblock_1 *sb = sbv;
+	int l = homehost ? strlen(homehost) : 0;
+
+	return (l > 0 && l < 32 &&
+		sb->set_name[l] == ':' &&
+		strncmp(sb->set_name, homehost, l) == 0);
+}
+
 static void uuid_from_super1(int uuid[4], void * sbv)
 {
 	struct mdp_superblock_1 *super = sbv;
@@ -421,16 +475,25 @@ static void getinfo_super1(struct mdinfo *info, void *sbv)
 	info->array.working_disks = working;
 }
 
-static int update_super1(struct mdinfo *info, void *sbv, char *update, char *devname, int verbose)
+static int update_super1(struct mdinfo *info, void *sbv, char *update,
+			 char *devname, int verbose,
+			 int uuid_set, char *homehost)
 {
+	/* NOTE: for 'assemble' and 'force' we need to return non-zero if any change was made.
+	 * For others, the return value is ignored.
+	 */
 	int rv = 0;
 	struct mdp_superblock_1 *sb = sbv;
 
 	if (strcmp(update, "force")==0) {
+		if (sb->events != __cpu_to_le64(info->events))
+			rv = 1;
 		sb->events = __cpu_to_le64(info->events);
 		switch(__le32_to_cpu(sb->level)) {
 		case 5: case 4: case 6:
 			/* need to force clean */
+			if (sb->resync_offset != ~0ULL)
+				rv = 1;
 			sb->resync_offset = ~0ULL;
 		}
 	}
@@ -466,8 +529,38 @@ static int update_super1(struct mdinfo *info, void *sbv, char *update, char *dev
 		/* make sure resync happens */
 		sb->resync_offset = ~0ULL;
 	}
-	if (strcmp(update, "uuid") == 0)
+	if (strcmp(update, "uuid") == 0) {
 		memcpy(sb->set_uuid, info->uuid, 16);
+		if (__le32_to_cpu(sb->feature_map)&MD_FEATURE_BITMAP_OFFSET) {
+			struct bitmap_super_s *bm;
+			bm = (struct bitmap_super_s*)(sbv+1024);
+			memcpy(bm->uuid, info->uuid, 16);
+		}
+	}
+	if (strcmp(update, "homehost") == 0 &&
+	    homehost) {
+		char *c;
+		update = "name";
+		c = strchr(sb->set_name, ':');
+		if (c)
+			strncpy(info->name, c+1, 31 - (c-sb->set_name));
+		else
+			strncpy(info->name, sb->set_name, 32);
+		info->name[33] = 0;
+	}
+	if (strcmp(update, "name") == 0) {
+		if (info->name[0] == 0)
+			sprintf(info->name, "%d", info->array.md_minor);
+		memset(sb->set_name, 0, sizeof(sb->set_name));
+		if (homehost &&
+		    strchr(info->name, ':') == NULL &&
+		    strlen(homehost)+1+strlen(info->name) < 32) {
+			strcpy(sb->set_name, homehost);
+			strcat(sb->set_name, ":");
+			strcat(sb->set_name, info->name);
+		} else
+			strcpy(sb->set_name, info->name);
+	}
 	if (strcmp(update, "_reshape_progress")==0)
 		sb->reshape_position = __cpu_to_le64(info->reshape_progress);
 
@@ -482,16 +575,20 @@ static __u64 event_super1(void *sbv)
 	return __le64_to_cpu(sb->events);
 }
 
-static int init_super1(struct supertype *st, void **sbp, mdu_array_info_t *info, unsigned long long size, char *name)
+static int init_super1(struct supertype *st, void **sbp, mdu_array_info_t *info,
+		       unsigned long long size, char *name, char *homehost)
 {
 	struct mdp_superblock_1 *sb = malloc(1024 + sizeof(bitmap_super_t));
 	int spares;
 	int rfd;
+	char defname[10];
 	memset(sb, 0, 1024);
 
-	if (info->major_version == -1)
+	if (info->major_version == -1) {
 		/* zeroing superblock */
+		*sbp = sb;
 		return 0;
+	}
 
 	spares = info->working_disks - info->active_disks;
 	if (info->raid_disks + spares  > 384) {
@@ -500,7 +597,6 @@ static int init_super1(struct supertype *st, void **sbp, mdu_array_info_t *info,
 		return 0;
 	}
 
-
 	sb->magic = __cpu_to_le32(MD_SB_MAGIC);
 	sb->major_version = __cpu_to_le32(1);
 	sb->feature_map = 0;
@@ -515,8 +611,19 @@ static int init_super1(struct supertype *st, void **sbp, mdu_array_info_t *info,
 	}
 	if (rfd >= 0) close(rfd);
 
+	if (name == NULL || *name == 0) {
+		sprintf(defname, "%d", info->md_minor);
+		name = defname;
+	}
 	memset(sb->set_name, 0, 32);
-	strcpy(sb->set_name, name);
+	if (homehost &&
+	    strchr(name, ':')== NULL &&
+	    strlen(homehost)+1+strlen(name) < 32) {
+		strcpy(sb->set_name, homehost);
+		strcat(sb->set_name, ":");
+		strcat(sb->set_name, name);
+	} else
+		strcpy(sb->set_name, name);
 
 	sb->ctime = __cpu_to_le64((unsigned long long)time(0));
 	sb->level = __cpu_to_le32(info->level);
@@ -559,6 +666,8 @@ static void add_to_super1(void *sbv, mdu_disk_info_t *dk)
 		*rp = 0xfffe;
 }
 
+static void locate_bitmap1(struct supertype *st, int fd, void *sbv);
+
 static int store_super1(struct supertype *st, int fd, void *sbv)
 {
 	struct mdp_superblock_1 *sb = sbv;
@@ -622,6 +731,15 @@ static int store_super1(struct supertype *st, int fd, void *sbv)
 	if (write(fd, sb, sbsize) != sbsize)
 		return 4;
 
+	if (sb->feature_map & __cpu_to_le32(MD_FEATURE_BITMAP_OFFSET)) {
+		struct bitmap_super_s *bm = (struct bitmap_super_s*)
+			(((char*)sb)+1024);
+		if (__le32_to_cpu(bm->magic) == BITMAP_MAGIC) {
+			locate_bitmap1(st, fd, sbv);
+			if (write(fd, bm, sizeof(*bm)) != sizeof(*bm))
+			    return 5;
+		}
+	}
 	fsync(fd);
 	return 0;
 }
@@ -650,7 +768,7 @@ static int write_init_super1(struct supertype *st, void *sbv,
 
 	sb->dev_number = __cpu_to_le32(dinfo->number);
 	if (dinfo->state & (1<<MD_DISK_WRITEMOSTLY))
-		sb->devflags |= WriteMostly1;
+		sb->devflags |= __cpu_to_le32(WriteMostly1);
 
 	if ((rfd = open("/dev/urandom", O_RDONLY)) < 0 ||
 	    read(rfd, sb->device_uuid, 16) != 16) {
@@ -669,7 +787,11 @@ static int write_init_super1(struct supertype *st, void *sbv,
 		if (memcmp(sb->set_uuid, refsb->set_uuid, 16)==0) {
 			/* same array, so preserve events and dev_number */
 			sb->events = refsb->events;
-			sb->dev_number = refsb->dev_number;
+			/* bugs in 2.6.17 and earlier mean the dev_number
+			 * chosen in Manage must be preserved
+			 */
+			if (get_linux_version() >= 2006018)
+				sb->dev_number = refsb->dev_number;
 		}
 		free(refsb);
 	}
@@ -767,8 +889,8 @@ static int compare_super1(void **firstp, void *secondv)
 		return 1;
 
 	if (!first) {
-		first = malloc(1024);
-		memcpy(first, second, 1024);
+		first = malloc(1024+sizeof(bitmap_super_t));
+		memcpy(first, second, 1024+sizeof(bitmap_super_t));
 		*firstp = first;
 		return 0;
 	}
@@ -791,7 +913,8 @@ static int load_super1(struct supertype *st, int fd, void **sbp, char *devname)
 	unsigned long long dsize;
 	unsigned long long sb_offset;
 	struct mdp_superblock_1 *super;
-
+	int uuid[4];
+	struct bitmap_super_s *bsb;
 
 
 	if (st->ss == NULL) {
@@ -850,7 +973,7 @@ static int load_super1(struct supertype *st, int fd, void **sbp, char *devname)
 	/*
 	 * Calculate the position of the superblock.
 	 * It is always aligned to a 4K boundary and
-	 * depeding on minor_version, it can be:
+	 * depending on minor_version, it can be:
 	 * 0: At least 8K, but less than 12K, from end of device
 	 * 1: At start of device
 	 * 2: 4K from start of device.
@@ -914,6 +1037,28 @@ static int load_super1(struct supertype *st, int fd, void **sbp, char *devname)
 		return 2;
 	}
 	*sbp = super;
+
+	/* Now check on the bitmap superblock */
+	if ((__le32_to_cpu(super->feature_map)&MD_FEATURE_BITMAP_OFFSET) == 0)
+		return 0;
+	/* Read the bitmap superblock and make sure it looks
+	 * valid.  If it doesn't clear the bit.  An --assemble --force
+	 * should get that written out.
+	 */
+	locate_bitmap1(st, fd, super);
+	if (read(fd, ((char*)super)+1024, sizeof(struct bitmap_super_s))
+	    != sizeof(struct bitmap_super_s))
+		goto no_bitmap;
+
+	uuid_from_super1(uuid, super);
+	bsb = (struct bitmap_super_s *)(((char*)super)+1024);
+	if (__le32_to_cpu(bsb->magic) != BITMAP_MAGIC ||
+	    memcmp(bsb->uuid, uuid, 16) != 0)
+		goto no_bitmap;
+	return 0;
+
+ no_bitmap:
+	super->feature_map = __cpu_to_le32(__le32_to_cpu(super->feature_map) & ~1);
 	return 0;
 }
 
@@ -1031,7 +1176,7 @@ add_internal_bitmap1(struct supertype *st, void *sbv,
 }
 
 
-void locate_bitmap1(struct supertype *st, int fd, void *sbv)
+static void locate_bitmap1(struct supertype *st, int fd, void *sbv)
 {
 	unsigned long long offset;
 	struct mdp_superblock_1 *sb;
@@ -1051,7 +1196,7 @@ void locate_bitmap1(struct supertype *st, int fd, void *sbv)
 	lseek64(fd, offset<<9, 0);
 }
 
-int write_bitmap1(struct supertype *st, int fd, void *sbv)
+static int write_bitmap1(struct supertype *st, int fd, void *sbv)
 {
 	struct mdp_superblock_1 *sb = sbv;
 	bitmap_super_t *bms = (bitmap_super_t*)(((char*)sb)+1024);
@@ -1092,6 +1237,7 @@ struct superswitch super1 = {
 	.detail_super = detail_super1,
 	.brief_detail_super = brief_detail_super1,
 #endif
+	.match_home = match_home1,
 	.uuid_from_super = uuid_from_super1,
 	.getinfo_super = getinfo_super1,
 	.update_super = update_super1,
